@@ -17,6 +17,7 @@
 import types
 import datetime
 import math
+from threading import Condition
 
 from pymongo.son import SON
 from pymongo.database import Database
@@ -26,11 +27,19 @@ from pymongo.binary import Binary
 from errors import CorruptGridFile
 from pymongo import ASCENDING
 
+# TODO we should use per-file reader-writer locks here instead,
+# for performance. Unfortunately they aren't in the Python standard library.
+_files_lock = Condition()
+_open_files = {}
+
+
 class GridFile(object):
     """A "file" stored in GridFS.
     """
     # TODO should be able to create a GridFile given a Collection object instead
     # of a database and collection name?
+    # TODO this whole file_spec thing is over-engineered. ought to be just
+    # filename.
     def __init__(self, file_spec, database, mode="r", collection="fs"):
         """Open a "file" in GridFS.
 
@@ -79,19 +88,29 @@ class GridFile(object):
             raise ValueError("mode must be one of ('r', 'w')")
 
         self.__collection = database[collection]
-        self.__collection.chunks.create_index([("files_id", ASCENDING), ("n", ASCENDING)])
+        self.__collection.chunks.ensure_index([("files_id", ASCENDING), ("n", ASCENDING)])
+
+        _files_lock.acquire()
 
         grid_file = self.__collection.files.find_one(file_spec)
         if grid_file:
             self.__id = grid_file["_id"]
         else:
             if mode == "r":
+                _files_lock.release()
                 raise IOError("No such file: %r" % file_spec)
             file_spec["length"] = 0
             file_spec["uploadDate"] = datetime.datetime.utcnow()
             file_spec.setdefault("chunkSize", 256000)
             self.__id = self.__collection.files.insert(file_spec)["_id"]
 
+        # we use repr(self.__id) here because we need it to be string and
+        # filename gets tricky with renaming. this is a hack.
+        while repr(self.__id) in _open_files:
+            _files_lock.wait()
+        _open_files[repr(self.__id)] = True
+        _files_lock.release()
+
         self.__mode = mode
         if mode == "w":
             self.__erase()
@@ -103,7 +122,7 @@ class GridFile(object):
     def __erase(self):
         """Erase all of the data stored in this GridFile.
         """
-        grid_file = self.__collection.files.find_one(self.__id)
+        grid_file = self.__collection.files.find_one({"_id": self.__id})
         grid_file["next"] = None
         grid_file["length"] = 0
         self.__collection.files.save(grid_file)
@@ -120,9 +139,9 @@ class GridFile(object):
 
     def __create_property(field_name, read_only=False):
         def getter(self):
-            return self.__collection.files.find_one(self.__id).get(field_name, None)
+            return self.__collection.files.find_one({"_id": self.__id}).get(field_name, None)
         def setter(self, value):
-            grid_file = self.__collection.files.find_one(self.__id)
+            grid_file = self.__collection.files.find_one({"_id": self.__id})
             grid_file[field_name] = value
             self.__collection.files.save(grid_file)
         if not read_only:
@@ -147,7 +166,7 @@ class GridFile(object):
         :Parameters:
           - `filename`: the new name for this GridFile
         """
-        grid_file = self.__collection.files.find_one(self.__id)
+        grid_file = self.__collection.files.find_one({"_id": self.__id})
         grid_file["filename"] = filename
         self.__collection.files.save(grid_file)
 
@@ -192,7 +211,7 @@ class GridFile(object):
         md5 = self.__collection.database()._command(SON([("filemd5", self.__id),
                                                          ("root", self.__collection.name())]))["md5"]
 
-        grid_file = self.__collection.files.find_one(self.__id)
+        grid_file = self.__collection.files.find_one({"_id": self.__id})
         grid_file["md5"] = md5
         grid_file["length"] = self.__position + len(self.__buffer)
         self.__collection.files.save(grid_file)
@@ -207,6 +226,12 @@ class GridFile(object):
             self.flush()
         self.__closed = True
 
+        _files_lock.acquire()
+        if repr(self.__id) in _open_files:
+            del _open_files[repr(self.__id)]
+            _files_lock.notifyAll()
+        _files_lock.release()
+
     def __assert_open(self, mode=None):
         if mode and self.mode != mode:
             raise ValueError("file must be open in mode %r" % mode)
